@@ -568,7 +568,6 @@ class LangUpload {
     \brief Ĥ줿ץɽμ
 
     \todo ؿ̾ѹꡢȤ
-    \todo enscript ʤȤνɲ
     \todo ե뤬ʤȤνˤĤˤ
   */
   function createProgramView($brows_name,
@@ -581,27 +580,52 @@ class LangUpload {
       return $out;
     }
     $rev = $this->rev;
+    $key_rev = $this->key. '-'. $rev;
 
     // !!! path ä̾ѹ롢
-    $path = 'data/'. $brows_name. '/p'. $this->key. '-'. $rev. '.txt';
+    $path = 'data/'. $brows_name. '/p'. $key_rev. '.txt';
 
-    // !!! enscript ʤȤե⡼ɤΤȤб
-    $cmd = $_SESSION['enscript_bin'].
-      ' --highlight=cpp --color --language html --toc --output - '. $path;
-    $stdout = @shell_exec($cmd);
-    $code = mb_detect_encoding($stdout, "auto", true);
-    $converted = ($code != false) ?
-      mb_convert_encoding($stdout, 'EUC-JP', $code) : $stdout;
+    // ϿγĥҤ
+    // !!! PEAR::Text_Highlighter Ȥ٤
+    $programs = $this->getProgramsInfo('data/'. $brows_name. '/programs.txt');
+    $suffix = strtolower($programs[$key_rev]['suffix']);
+    switch ($ext) {
+    default:
+      $enscript_suffix = 'cpp';
+      break;
+    }
 
-    // !!! preg_match ֤
-    if (ereg('(<PRE>.+</PRE>)', $converted, $prog)) {
-      $out .= $prog[1];
+    if ($suffix == 'php') {
+      // !!! EUC-JP ʳн褹٤ʤ...
+      highlight_string(join(file($path)));
     } else {
-      $lines = @file($path);
-      if (is_array($lines)) {
-	$out .= '<pre>'. join($lines). '</pre>';
+      // !!! enscript ʤȤե⡼ɤΤȤб
+      $cmd = $_SESSION['enscript_bin'].
+	' --highlight='. $enscript_suffix. ' --color --language html --toc --output - '. $path;
+      $stdout = @shell_exec($cmd);
+      if (isset($programs[$key_rev]['code'])) {
+	$code = $programs[$key_rev]['code'];
       } else {
-	$out .= $no_program_message;
+	$code = mb_detect_encoding($stdout, "auto", true);
+	$programs[$key_rev]['code'] = $code;
+
+	// programs.txt ι
+	$this->updateProgramsList($programs);
+      }
+
+      $converted = ($code != false) ?
+	mb_convert_encoding($stdout, 'EUC-JP', $code) : $stdout;
+
+      // !!! preg_match ֤
+      if (ereg('(<PRE>.+</PRE>)', $converted, $prog)) {
+	$out .= $prog[1];
+      } else {
+	$lines = @file($path);
+	if (is_array($lines)) {
+	  $out .= '<pre>'. join($lines). '</pre>';
+	} else {
+	  $out .= $no_program_message;
+	}
       }
     }
     return $out;
@@ -1190,5 +1214,61 @@ class LangUpload {
       fclose($fp);
     }
   }
+
+
+  // ץγĥҤ
+  function getProgramsInfo($file) {
+
+    $programs = array();
+
+    $fp = @fopen($file, 'r');
+    if ($fp == false) {
+      return $programs;
+    }
+    flock($fp, LOCK_SH);
+    while (! feof($fp)) {
+      $line = fgets($fp);
+      if (preg_match("/p([\d]+-[\d]+-[\d]+)\.txt[ \t]+(.+)/",
+		     $line, $matches)) {
+	$key = $matches[1];
+	$original = $matches[2];
+	$program_name = strtok($original, "\t ");
+	$jp_code = strtok("\t\n ");
+	if (preg_match("/.+?\.(.+)/", $program_name, $matches)) {
+	  $suffix = $matches[1];
+	} else {
+	  $suffix = 'txt';
+	}
+	$programs[$key]['name'] = $program_name;
+	$programs[$key]['suffix'] = $suffix;
+
+	if ($jp_code != '') {
+	  $programs[$key]['code'] = $jp_code;
+	}
+      }
+    }
+    fclose($fp);
+
+    return $programs;
+  }
+
+
+  // ץեι
+  function updateProgramsList($programs) {
+    $dir = 'data/'. $_SESSION['login_name']. '/';
+    $fp = fopen($dir. 'programs.txt', 'w');
+    if ($fp != false) {
+      flock($fp, LOCK_EX);
+      foreach ($programs as $key => $value) {
+
+	$line = 'p'. $key. '.txt'. "\t". $value['name'];
+	if (isset($value['code'])) {
+	  $line .= "\t". $value['code'];
+	}
+	fwrite($fp, $line. "\n");
+      }
+      fclose($fp);
+    }
+  }
 };
 ?>
